@@ -16,7 +16,7 @@ from io3d import cachefile
 import json
 import time
 import platform
-from typing import List, Union
+from typing import List, Union, Optional
 
 # import PyQt5.QtWidgets
 # print("start 3")
@@ -29,6 +29,7 @@ from pyqtgraph.parametertree import Parameter, ParameterTree
 import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
+
 # matplotlib.use('Qt5Agg')
 
 # print("start 6")
@@ -43,13 +44,11 @@ import scaffan.slide_segmentation
 from scaffan.pyqt_widgets import BatchFileProcessingParameter
 from scaffan.image_intensity_rescale_pyqtgraph import RescaleIntensityPercentilePQG
 from . import sni_prediction
+from . import lobule_quality_estimation_cnn
 
 
 class Scaffan:
-    def __init__(
-            self,
-            whole_scan_margin=-0.0
-    ):
+    def __init__(self, whole_scan_margin=-0.0):
         """
 
         :param whole_scan_margin: negative value makes the the area for automatic lobuli selection smaller. It is used in
@@ -79,7 +78,12 @@ class Scaffan:
         self.skeleton_analysis = scaffan.skeleton_analysis.SkeletonAnalysis()
         self.evaluation = scaffan.evaluation.Evaluation()
         self.intensity_rescale = RescaleIntensityPercentilePQG()
-        self.slide_segmentation = scaffan.slide_segmentation.ScanSegmentation(report=self.report)
+        self.slide_segmentation = scaffan.slide_segmentation.ScanSegmentation(
+            report=self.report
+        )
+        self.lobule_quality_estimation_cnn = lobule_quality_estimation_cnn.LobuleQualityEstimationCNN(
+            report=self.report
+        )
         # self.slide_segmentation.report = self.report
 
         # self.lobulus_processing.set_report(self.report)
@@ -108,20 +112,19 @@ class Scaffan:
                         "name": "Lobulus Selection Method",
                         "type": "list",
                         "value": "Auto",
-
-                        "values": ["Color","Manual", "Auto"],
-                            # "Color": "Color",
-                            # "Manual": "Manual",
-                            # "Auto": "Auto",
+                        "values": ["Color", "Manual", "Auto"],
+                        # "Color": "Color",
+                        # "Manual": "Manual",
+                        # "Auto": "Auto",
                         # },
                         "tip": "Auto: select lobulus based on Scan Segmentation.\n"
-                               "Color: based on annotation color.\n"
-                               "Manual: manually pick the lobule. \n\n" +
-                       "The annotations from `.ndpi` files are automatically imported.\n " +
-                       "The annotattion for other image format are expected to be done in ImageJ ROI Manager.\n"
-                       "File with annotations for is expected to be in the same dir and same name "
-                       "with file extension `.roi.zip`. \n"
-                       "Color of segmentation can be part of the polyon name ('my annotation #00FF00')"
+                        "Color: based on annotation color.\n"
+                        "Manual: manually pick the lobule. \n\n"
+                        + "The annotations from `.ndpi` files are automatically imported.\n "
+                        + "The annotattion for other image format are expected to be done in ImageJ ROI Manager.\n"
+                        "File with annotations for is expected to be in the same dir and same name "
+                        "with file extension `.roi.zip`. \n"
+                        "Color of segmentation can be part of the polyon name ('my annotation #00FF00')",
                     },
                     # {
                     #     "name": "Lobulus Selection Method",
@@ -133,8 +136,8 @@ class Scaffan:
                         "name": "Annotation Color",
                         "type": "list",
                         "tip": "Select lobulus based on annotation color. "
-                               "Color of the annotation from ImageJ is expected to be part of the polyon name ('my annotation #00FF00')"
-                               "Skipped if Automatic Lobulus Selection Method is used.",
+                        "Color of the annotation from ImageJ is expected to be part of the polyon name ('my annotation #00FF00')"
+                        "Skipped if Automatic Lobulus Selection Method is used.",
                         "values": {
                             "None": None,
                             "White": "#FFFFFF",
@@ -225,6 +228,7 @@ class Scaffan:
                     self.lobulus_processing.parameters,
                     self.skeleton_analysis.parameters,
                     self.glcm_textures.parameters,
+                    self.lobule_quality_estimation_cnn.parameters,
                     {
                         "name": "Whole Scan Margin",
                         "type": "float",
@@ -270,8 +274,8 @@ class Scaffan:
             None,
             "Select Input File",
             directory=default_dir,
-            filter="Images (*.ndpi *.czi *.tif *.tiff);;NanoZoomer Digital Pathology Image (*.ndpi);;" +
-                   "Zeiss Image format for microscopes (*.czi);;Tiff image (*.tiff *.tif)",
+            filter="Images (*.ndpi *.czi *.tif *.tiff);;NanoZoomer Digital Pathology Image (*.ndpi);;"
+            + "Zeiss Image format for microscopes (*.czi);;Tiff image (*.tiff *.tif)",
         )
         self.set_input_file(fn)
 
@@ -404,6 +408,8 @@ class Scaffan:
         logger.debug(f"report output dir: {self.report.outputdir}")
         fn_spreadsheet = self.parameters.param("Output", "Common Spreadsheet File")
         self.report.additional_spreadsheet_fn = str(fn_spreadsheet.value())
+        if self.get_parameter("Processing;Quality Estimation CNN") ==  True:
+            self.lobule_quality_estimation_cnn.init()
 
     def set_annotation_color_selection(
         self, color: str, override_automatic_lobulus_selection=True
@@ -452,10 +458,22 @@ class Scaffan:
             # mainapp.set_annotation_color_selection("#FF00FF")
             # mainapp.set_annotation_color_selection("#FF0000")
             #             mainapp.set_annotation_color_selection("#FFFF00")
-            self.set_parameter("Input;Lobulus Selection Method", "Auto")
-            self.set_parameter("Processing;Skeleton Analysis", False)
-            self.set_parameter("Processing;Texture Analysis", False)
-            self.set_parameter("Processing;Lobulus Segmentation", False)
+            paramlist = [
+                ("Input;Lobulus Selection Method", "None"),
+                ("Processing;Skeleton Analysis", False),
+                ("Processing;Texture Analysis", False),
+                ("Processing;Lobulus Segmentation", False),
+                ("Processing;Scan Segmentation;Run Prediction", False)
+            ]
+            prev_values = []
+            for key, val in paramlist:
+                prev_values.append(self.get_parameter(key))
+                self.set_parameter(key, val)
+
+            # self.set_parameter("Processing;Skeleton Analysis", False)
+            # self.set_parameter("Processing;Texture Analysis", False)
+            # self.set_parameter("Processing;Lobulus Segmentation", False)
+            # self.set_parameter("Processing;Scan Segmentation;Run Prediction", False)
             if i == 0:
                 if clean_before_training is not None:
                     self.set_parameter(
@@ -471,23 +489,48 @@ class Scaffan:
             # mainapp.start_gui(qapp=qapp)
             self.run_lobuluses()
 
-    def run_lobuluses(self, event=None):
-        self.init_run()
-        annotation_ids = None
-        # if color is None:
+            # reset values to prev values before function run
+            for wanted, val in zip(paramlist, prev_values):
+                key, wanted_val = wanted
+                self.set_parameter(key, val)
+
+    def _prepare_annoataion_ids_for_run_lobuluses(self, annotation_ids:Optional[list]=None):
         pcolor = self.parameters.param("Input", "Annotation Color")
         color = pcolor.value()
+        automatic_lobulus_selection = self.parameters.param(
+            "Input", "Lobulus Selection Method"
+        ).value()
+        logger.debug(f"Lobulus Selection={automatic_lobulus_selection}")
+        if annotation_ids is None:
+
+            if automatic_lobulus_selection == "Color":
+                annotation_ids = self.anim.get_annotations_by_color(
+                    color,
+                    raise_exception_if_not_found=self.raise_exception_if_color_not_found,
+                )
+            # elif automatic_lobulus_selection and not run_slide_segmentation:
+            elif automatic_lobulus_selection == "Manual":
+                # on training we dont want to do the prediction and we set
+                annotation_ids = self.manual_select()
+            elif automatic_lobulus_selection == "None":
+                annotation_ids = []
+            else:
+                logger.error(f"Unknown Lobulus Selection Method: {automatic_lobulus_selection}")
+        return annotation_ids, automatic_lobulus_selection
+
+    def run_lobuluses(self, event=None, seeds_mm:Optional[list]=None):
+        """
+        :param event:
+        :param seeds_mm: If not none, and Lobulus Selection Method is set to
+        :return:
+        """
+        annotation_ids = None
+        self.init_run()
+        if seeds_mm:
+            logger.debug(f"seeds_mm={seeds_mm}")
+            _, annotation_ids = self.prepare_circle_annotations_from_seeds_mm(centers_mm=seeds_mm)
+
         self.report.level = self.parameters.param("Processing", "Report Level").value()
-        # print("Color ", color)
-        # fnparam = self.parameters.param("Input", "File Path")
-        # from .image import AnnotatedImage
-        # path = self.parameters.param("Input", "File Path")
-        # anim = AnnotatedImage(path.value())
-        # if color is None:
-        #     color = list(self.anim.colors.keys())[0]
-        # print(self.anim.colors)
-        # if annotation_ids is None:
-        #     logger.error("No color selected")
 
         show = self.parameters.param("Processing", "Show").value()
         self.report.set_show(show)
@@ -495,18 +538,10 @@ class Scaffan:
         run_slide_segmentation = self.parameters.param(
             "Processing", "Scan Segmentation"
         ).value()
-        automatic_lobulus_selection = self.parameters.param(
-            "Input", "Lobulus Selection Method"
-        ).value()
-        logger.debug(f"Lobulus Selection={automatic_lobulus_selection}")
-        if automatic_lobulus_selection == "Color":
-            annotation_ids = self.anim.get_annotations_by_color(
-                color,
-                raise_exception_if_not_found=self.raise_exception_if_color_not_found,
-            )
-        # elif automatic_lobulus_selection and not run_slide_segmentation:
-        if automatic_lobulus_selection == "Manual":
-            annotation_ids = self.manual_select()
+        annotation_ids, automatic_lobulus_selection = self._prepare_annoataion_ids_for_run_lobuluses(annotation_ids)
+
+        # if automatic_lobulus_selection == "use annotation_ids": ...
+
         if run_slide_segmentation:
             # fn_input = self.parameters.param("Input", "File Path").value()
 
@@ -519,7 +554,6 @@ class Scaffan:
                 self.slide_segmentation.add_biggest_to_annotations()
                 annotation_ids = self.slide_segmentation.ann_biggest_ids
 
-
         if annotation_ids is None:
             raise NoLobulusSelectionUsedError
         logger.debug("Annotation IDs: {}".format(annotation_ids))
@@ -623,6 +657,9 @@ class Scaffan:
         run_texture_analysis = self.parameters.param(
             "Processing", "Texture Analysis"
         ).value()
+        run_lobule_quality_estimation = self.parameters.param(
+            "Processing", "Quality Estimation CNN"
+        ).value()
         logger.debug("before skeleton analysis")
         if run_skeleton_analysis:
             self.skeleton_analysis.skeleton_analysis(show=show)
@@ -634,6 +671,13 @@ class Scaffan:
                 lobulus_segmentation=self.lobulus_processing.lobulus_mask,
             )
             self.glcm_textures.run()
+        if run_lobule_quality_estimation:
+            self.lobule_quality_estimation_cnn.set_input_data(
+                view=self.lobulus_processing.view,
+                annotation_id=annotation_id,
+                lobulus_segmentation=self.lobulus_processing.lobulus_mask,
+            )
+            self.lobule_quality_estimation_cnn.run()
         logger.trace("after texture analysis")
         t1 = time.time()
         ann_center = self.anim.get_annotation_center_mm(annotation_id)
@@ -662,19 +706,75 @@ class Scaffan:
         # Copy all parameters to table
         self.report.finish_actual_row()
 
+    def get_preview_view(self):
+        full_view = self.anim.get_view(
+            location=[0, 0], level=0, size_on_level=self.anim.get_slide_size()[::-1]
+        )
+        pxsz_mm = float(self.get_parameter("Processing;Preview Pixelsize")) * 1000
+        view_corner = full_view.to_pixelsize(pixelsize_mm=[pxsz_mm, pxsz_mm])
+        return view_corner
+
+    def get_preview(self):
+        view_corner = self.get_preview_view()
+        # logger.debug(
+        #     f"Manual selection1, view.loc={full_view.region_location}, view.size={full_view.region_size_on_level}, pxsz={full_view.region_pixelsize}"
+        # )
+        logger.debug(
+            f"Manual selection2, view.loc={view_corner.region_location}, view.size={view_corner.region_size_on_level}, pxsz={view_corner.region_pixelsize}"
+        )
+        img = view_corner.get_region_image(as_gray=False)
+        return view_corner, img
+
+    def prepare_circle_annotations_from_seeds_mm(self, centers_mm):
+        pxsz_mm = float(self.get_parameter("Processing;Preview Pixelsize")) * 1000
+        centers_mm = list(np.asarray(centers_mm) / pxsz_mm)
+
+        view_corner = self.get_preview_view()
+        return self.prepare_circle_annotations_from_points_px_in_preview(view_corner, centers_mm)
+
+    def prepare_circle_annotations_from_points_px_in_preview(self, view_corner, centers_px_view):
+
+        x_px_view, y_px_view = zip(*list(centers_px_view))
+        logger.debug(x_px_view)
+        pts_glob_px = view_corner.coords_view_px_to_glob_px(np.asarray(x_px_view), np.asarray(y_px_view))
+        centers_px = list(zip(*pts_glob_px))
+        logger.debug(f"Manual selection5, centers_px_global={centers_px}")
+        r_mm = (
+                float(self.get_parameter("Processing;Scan Segmentation;Annotation Radius"))
+                * 1000
+        )
+
+        ann_ids, _ = scaffan.slide_segmentation.add_circle_annotation(
+            view_corner, centers_px, annotations=self.anim.annotations, r_mm=r_mm
+        )
+        view_corner.set_annotations(self.anim.annotations)
+        view_corner.adjust_annotation_to_image_view()
+        return view_corner, ann_ids
+
     def manual_select(self):
         logger.debug("Manual selection")
         # full_view = self.anim.get_full_view()
-        full_view = self.anim.get_view(location=[0,0], level=0, size_on_level=self.anim.get_slide_size()[::-1])
-        pxsz_mm = float(self.get_parameter("Processing;Preview Pixelsize")) * 1000
-        view_corner = full_view.to_pixelsize(pixelsize_mm=[pxsz_mm, pxsz_mm])
-        logger.debug(f"Manual selection1, view.loc={full_view.region_location}, view.size={full_view.region_size_on_level}, pxsz={full_view.region_pixelsize}")
-        logger.debug(f"Manual selection2, view.loc={view_corner.region_location}, view.size={view_corner.region_size_on_level}, pxsz={view_corner.region_pixelsize}")
-        img = view_corner.get_region_image(as_gray=False)
+        view_corner, img = self.get_preview()
+        # full_view = self.anim.get_view(
+        #     location=[0, 0], level=0, size_on_level=self.anim.get_slide_size()[::-1]
+        # )
+        # pxsz_mm = float(self.get_parameter("Processing;Preview Pixelsize")) * 1000
+        # view_corner = full_view.to_pixelsize(pixelsize_mm=[pxsz_mm, pxsz_mm])
+        # logger.debug(
+        #     f"Manual selection1, view.loc={full_view.region_location}, view.size={full_view.region_size_on_level}, pxsz={full_view.region_pixelsize}"
+        # )
+        # logger.debug(
+        #     f"Manual selection2, view.loc={view_corner.region_location}, view.size={view_corner.region_size_on_level}, pxsz={view_corner.region_pixelsize}"
+        # )
+        # img = view_corner.get_region_image(as_gray=False)
         plt.ioff()
         fig = plt.figure(figsize=(12, 8))
-        fig.canvas.set_window_title("Select lobules. Left/Middle/Right Mouse Button: add/quit/remove")
-        logger.debug(f"Manual selection2 backend={matplotlib.get_backend()}, ion={matplotlib.is_interactive()}, img.shape={img.shape}, img.max={np.max(img)}")
+        fig.canvas.set_window_title(
+            "Select lobules. Left/Middle/Right Mouse Button: add/quit/remove"
+        )
+        logger.debug(
+            f"Manual selection2 backend={matplotlib.get_backend()}, ion={matplotlib.is_interactive()}, img.shape={img.shape}, img.max={np.max(img)}"
+        )
         plt.imshow(img)
         # plt.ginput(1)
         # plt.axis("image")
@@ -685,30 +785,24 @@ class Scaffan:
         plt.close(fig)
         logger.debug(f"Manual selection5, centers_px={points_px}")
 
-
-
         # points_px = np.asarray(centers_px)
         from scaffan.image import get_offset_px, get_pixelsize
 
         # offset_px = get_offset_px(self.anim)
         # pixelsize, pixelunit = get_pixelsize(self.anim, requested_unit="mm")
-        x_px_view =  points_px[:,0].flatten()
-        y_px_view =  points_px[:,1].flatten()
+        x_px_view = points_px[:, 0].flatten()
+        y_px_view = points_px[:, 1].flatten()
         # x_px = (x_px_view*view_corner.zoom[0] + offset_px[0])
         # y_px = (y_px_view*view_corner.zoom[1] + offset_px[1])
         # pts_glob_px = [x_px, y_px]
+        xy_px_view = zip(x_px_view, y_px_view)
 
-        pts_glob_px = view_corner.coords_view_px_to_glob_px(
-            x_px_view, y_px_view
-        )
-        centers_px = list(zip(*pts_glob_px))
-        logger.debug(f"Manual selection5, centers_px_global={centers_px}")
+        pxsz_mm = float(self.get_parameter("Processing;Preview Pixelsize")) * 1000
+        seeds_mm = list(np.asarray(xy_px_view) / pxsz_mm)
+        logger.debug(f"seeds_mm={seeds_mm}")
         # centers_px = list(zip(*pts_glob_px))
-        r_mm = float(self.get_parameter("Processing;Scan Segmentation;Annotation Radius")) * 1000
-
-        ann_ids, _ = scaffan.slide_segmentation.add_circle_annotation(view_corner, centers_px, annotations=self.anim.annotations, r_mm=r_mm)
-        view_corner.set_annotations(self.anim.annotations)
-        view_corner.adjust_annotation_to_image_view()
+        view_corner, ann_ids = self.prepare_circle_annotations_from_points_px_in_preview(
+            view_corner, centers_px_view=xy_px_view)
         # logger.debug(f"annotations={self.anim.annotations}")
 
         logger.debug("annotation selected")
